<?php

namespace App\Http\Controllers;

use App\Models\City;
use App\Models\Patient;
use App\Models\Problem;
use App\Models\Schedule;
use Illuminate\Http\Request;
use App\Models\State;
use \Yajra\Datatables\Datatables;
use App\MsgApp;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;

class PatientController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        if ($request->ajax()) {
            $data = Patient::select('*');
            return Datatables::of($data)
                ->addIndexColumn()
                ->editColumn('state_id', function ($row) {
                    return ($row->state_id) ? $row->state->name : '';
                })
                ->editColumn('dos', function ($row) {
                    return date("d M Y", strtotime($row->dos));
                })
                ->editColumn('city_id', function ($row) {
                    return ($row->city_id) ? $row->city->name : '';
                })
                ->addColumn('action', function ($row) {
                    return '<a href="' . route('patient.show', $row->id) . '" class="edit btn btn-success btn-sm mr-2">
                    <en class="la la-eye icon-md"></en>
                    </a>';
                })
                ->rawColumns(['action'])
                ->make(true);
        }
        return view('patient.index');
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $states = State::where('status', 'Active')->get();
        $cities = [];
        return view('patient.create', compact('states', 'cities'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $validator = Validator::make(
            $request->all(),
            [
                'name' => 'required|string|max:20',
                'email' => 'nullable|regex:/^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,15})$/',
                'mobile' => 'nullable|regex:/^[6-9][0-9]{9}$/',
                'dob' => 'nullable|date',
                'age' => 'required',
                'address' => 'nullable|regex:/^[#.0-9a-zA-Z\s,-\/]+$/',
                'pin_code' => 'nullable|regex:/^[1-9][0-9]{5}$/',
                'state_id' => Str::replace(MsgApp::TABLE_NAME, 'states', MsgApp::REQ_EXISTS_NULL),
                'city_id' => Str::replace(MsgApp::TABLE_NAME, 'cities', MsgApp::REQ_EXISTS_NULL),
                'dob' => 'nullable|date',
                'problems' => 'required',
                'treatment' => 'required',
                'remark' => 'required',
                'schedule' => 'required|array',
                'sitting' => 'required',
            ],
            [
                'required' => ':attribute is required.',
                'regex' => ':attribute is invalid.',
            ],
            ['name' => 'Name']
        );
        if ($validator->fails()) {
            return Redirect::back()->withErrors($validator)->withInput($request->all());
        }
        $patient = Patient::saveData($request);
        $problem = Problem::saveData($request, $patient->id);
        Schedule::prepareData($request, $patient->dos, $patient->id, $problem->id);
        return Redirect('patient')->with("success", MsgApp::SUCCESS_ADDED);
    }

    /**
     * Display the specified resource.
     */
    public function show($id)
    {
        $patient = Patient::find($id);
        return view('patient.view', compact('patient'));
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit($id)
    {
        $patient = Patient::find($id);
        $states = State::where('status', 'Active')->get();
        $cities = City::where('state_id', $patient->state_id)->get();

        return view('patient.edit', compact('patient', 'states', 'cities'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $id)
    {
        $validator = Validator::make(
            $request->all(),
            [
                'name' => 'required|string|max:20',
                'email' => 'nullable|regex:/^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,15})$/',
                'mobile' => 'nullable|regex:/^[6-9][0-9]{9}$/|unique:patients,phone,' . $id,
                'dob' => 'nullable|date',
                'age' => 'required',
                'address' => 'nullable|regex:/^[#.0-9a-zA-Z\s,-\/]+$/',
                'pin_code' => 'nullable|regex:/^[1-9][0-9]{5}$/',
                'state_id' => Str::replace(MsgApp::TABLE_NAME, 'states', MsgApp::REQ_EXISTS_NULL),
                'city_id' => Str::replace(MsgApp::TABLE_NAME, 'cities', MsgApp::REQ_EXISTS_NULL),
            ],
            [
                'required' => ':attribute is required.',
                'regex' => ':attribute is invalid.',
            ],
            ['name' => 'Name']
        );
        if ($validator->fails()) {
            return Redirect::back()->withErrors($validator)->withInput($request->all());
        }
        $patient = Patient::saveData($request, $id);
        return Redirect('patient')->with("success", MsgApp::SUCCESS_UPD);
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy($id)
    {
        //
    }
    public function addSchedule(Request $request)
    {
        // Schedule::saveData($request, $request->schedule_id);
        return redirect()->route('patient.show', [$request->schedule_id])->with("success", MsgApp::SUCCESS_ADDED);
    }

    function treatment(Request $request, $id)
    {
        if ($request->ajax()) {
            $data = Problem::select('*')->where('patient_id', $id);
            return Datatables::of($data)
                ->addIndexColumn()
                ->editColumn('created_at', function ($row) {
                    return $row->created_at->format('d M Y');
                })

                ->addColumn('action', function ($row) {
                    return '<a href="' . route('patient.treatment.edit', [$row->patient_id, $row->id]) . '" class="edit btn btn-success btn-sm mr-2"><en class="la la-edit icon-md"></en></a>
                    <a href="javascript:;" class="edit btn btn-success btn-sm mr-2" onclick="viewSchedule(' . $row->id . ')">
                    <en class="la la-calendar-o icon-md"></en>
                    </a>';
                })
                ->rawColumns(['action'])
                ->make(true);
        }
    }
    function scheduleAdd($pid, $tid, $schedule_date)
    {
        $scheduleDay = date('l', strtotime($schedule_date));
        Schedule::saveData($scheduleDay, $schedule_date, $pid, $tid);
        return $this->schedule($tid);
    }

    function delSchedule($tid, $id){
        Schedule::where('id', $id)->delete();
        return $this->schedule($tid);
    }
    function schedule($id)
    {
        $schedules = Schedule::where('problem_id', $id)->orderBy('schedule_date', 'asc')->get();
        $sts = 'Yes';
        return view('schedule.table', compact('schedules', 'sts'));
    }
    function getSchedule($scheduleDate){
        $schedules = Schedule::where('schedule_date', $scheduleDate)->orderBy('schedule_date', 'asc')->get();
        $sts = 'No';
        return view('schedule.table', compact('schedules', 'sts'));
    }
}
